// rel_dat.c - Definicije paketa za rad sa relativnim datotekama.

#include "rel_dat.h"
#include <stdlib.h>

Rel_dat *rd_pravi(const char *ime, int duz) {// Stvaranje nove datoteke.
  FILE *dat; Rel_dat *rel_dat = NULL;
  if ( (dat = fopen(ime, "rb" )) != NULL         ||
       (dat = fopen(ime, "wb+")) == NULL         ||
       fwrite(&duz, sizeof(int), 1, dat) < 1     ||
       (rel_dat = malloc(sizeof(Rel_dat))) == NULL
     ) { if (dat) fclose(dat); return NULL; }
  rel_dat->dat  = dat; rel_dat->duz  = duz; rel_dat->br_zap = 0;
  return rel_dat;
}
 
Rel_dat *rd_otvori(const char *ime) {        // Otvaranje stare datoteke.
  FILE *dat; Rel_dat *rel_dat;
  int duz; long vel;
  if ( (dat = fopen(ime, "rb+")) == NULL         ||
       fread(&duz, sizeof(int), 1, dat) < 1      ||
       fseek(dat, 0L, SEEK_END) != 0             ||
       (vel = ftell(dat)) == -1L                 ||
       (vel - sizeof(int)) % duz != 0            ||
       (rel_dat = malloc(sizeof(Rel_dat))) == NULL
     ) { fclose(dat); return NULL; }
  rel_dat->dat  = dat;
  rel_dat->duz  = duz;
  rel_dat->br_zap = (vel - sizeof(int)) / duz;
  return rel_dat;
}

int rd_pisi(Rel_dat *dat, long k, const void *zap) { // Upisivanje zapisa.
  if (fseek(dat->dat,
             (long)(k-1)*dat->duz+sizeof(int),
             SEEK_SET
           ) != 0                            ||
       fwrite(zap, dat->duz, 1, dat->dat) < 1
     ) return EOF;
  if (k > dat->br_zap) dat->br_zap = k;
  return 0;
}

int rd_citaj(Rel_dat *dat, long k, void *zap) {      // Čitanje zapisa.
  if ( k > dat->br_zap                           ||
       fseek(dat->dat,
             (long)(k-1)*dat->duz+sizeof(int),
             SEEK_SET
            ) != 0                            ||
       fread(zap, dat->duz, 1, dat->dat) < 1
     ) return EOF;
  return 0;
}

long rd_broj_zap(Rel_dat *dat) {             // Broj zapisa u datoteci.
  return dat->br_zap;
}

int rd_zatvori(Rel_dat *dat) {               // Zatvaranje datoteke.
  if (!dat || !dat->dat) return EOF;
  fclose(dat->dat);
  free(dat);
  return 0;
}
